/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.binding.xmpp;

import java.util.List;
import java.util.Properties;
import java.util.logging.Logger;

import org.objectweb.petals.binding.xmpp.listeners.SessionDescriptor;
import org.objectweb.petals.binding.xmpp.listeners.XmppSessionManager;
import org.objectweb.petals.component.framework.PEtALSCDKException;
import org.objectweb.petals.component.framework.su.ServiceUnitListener;
import org.objectweb.petals.component.framework.util.StringHelper;
import org.objectweb.petals.jbi.descriptor.Consumes;
import org.objectweb.petals.jbi.descriptor.JBIDescriptor;
import org.objectweb.petals.jbi.descriptor.Provides;

public class XmppSUListener implements ServiceUnitListener {

    /**
     * The XMPP component context (configuration values)
     */
    private XmppComponentContext xmppContext;

    private XmppSessionManager xmppSessionmanager;
    
    private Logger logger;

    public XmppSUListener(final XmppSessionManager xmppSessionmanager,final XmppComponentContext xmppContext, final Logger logger) {
        this.xmppSessionmanager=xmppSessionmanager;
        this.xmppContext = xmppContext;
        this.logger = logger;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.petals.component.common.su.ServiceUnitListener#onSUDeployed(java.lang.String,
     *      java.lang.String,
     *      org.objectweb.petals.tools.jbicommon.descriptor.JBIDescriptor)
     */
    public void onSUDeployed(final String serviceUnitName,
            final String suRootPath, final JBIDescriptor descriptor)
            throws PEtALSCDKException {

        String username="";
        String hostname="";
        
        this.logger.info("su deployed -"+suRootPath);

        List<Provides> providesList = descriptor.getServices().getProvides();
        for (Provides provide : providesList) {
            org.objectweb.petals.component.framework.util.Extensions cdkExtensions = new org.objectweb.petals.component.framework.util.Extensions(
                    provide.getExtensions());

            if (cdkExtensions != null) {
                Properties props = cdkExtensions.getPetalsExtensions();
                username = props.getProperty(XmppBCImpl.USERNAME_PATHELEMENT);
                hostname = props.getProperty(XmppBCImpl.HOSTNAME_PATHELEMENT);
            }
        }

        List<Consumes> consumesList = descriptor.getServices().getConsumes();
        for (Consumes consume : consumesList) {
            org.objectweb.petals.component.framework.util.Extensions cdkExtensions = new org.objectweb.petals.component.framework.util.Extensions(
                    consume.getExtensions());

            if (cdkExtensions != null) {
                Properties props = cdkExtensions.getPetalsExtensions();
                username = props.getProperty(XmppBCImpl.USERNAME_PATHELEMENT);
                hostname = props.getProperty(XmppBCImpl.HOSTNAME_PATHELEMENT);
            }
        }
        
        if ((!(StringHelper.isNullOrEmpty(username))) && (!(StringHelper.isNullOrEmpty(hostname)))) {
            this.xmppContext.addSuRootPath(username + "@"+hostname, suRootPath);
        }
        
    }
 
    public void onSUStarted(final String arg0) throws PEtALSCDKException {

    }

    public void onSUStopped(final String arg0) throws PEtALSCDKException {

    }

    public void onSUUndeployed(final String arg0, final String arg1, final JBIDescriptor descriptor)
            throws PEtALSCDKException {
        String username="";
        String hostname="";
        String resource="";
        
        List<Consumes> consumesList = descriptor.getServices().getConsumes();
        for (Consumes consume : consumesList) {
            org.objectweb.petals.component.framework.util.Extensions cdkExtensions = new org.objectweb.petals.component.framework.util.Extensions(
                    consume.getExtensions());

            if (cdkExtensions != null) {
                Properties props = cdkExtensions.getPetalsExtensions();
                username = props.getProperty(XmppBCImpl.USERNAME_PATHELEMENT);
                hostname = props.getProperty(XmppBCImpl.HOSTNAME_PATHELEMENT);
                resource = props.getProperty(XmppBCImpl.RESOURCE_PATHELEMENT);
            }
        }
        
        String jid = StringHelper.isNullOrEmpty(username) ? "" : username
                + (StringHelper.isNullOrEmpty(hostname) ? "" : "@"
                        + hostname
                        + (StringHelper.isNullOrEmpty(resource) ? "" : "/"
                                + resource));
        
        if ((!(StringHelper.isNullOrEmpty(username))) && (!(StringHelper.isNullOrEmpty(hostname)))) {
           SessionDescriptor sd = this.xmppSessionmanager.retrieveSessionIfExists(jid);
           if ((sd != null) && (sd.getXmppConnection() != null) && (sd.getXmppConnection().isConnected())) {
               sd.setListening(false);
           }
        }

    }

}
