/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.binding.xmpp.listeners;

import java.util.Timer;
import java.util.TimerTask;
import java.util.logging.Logger;

import org.jivesoftware.smack.XMPPConnection;

/**
 * this class is used to check inactivity for a given connection.If the
 * inactivity duration is too long, it closes the connection
 * @author mcarpentier - EBM Websourcing
 */
public class InactivityChecker {
    /**
     * logger
     */
    protected Logger log;

    /**
     * timer used to check inactivity
     */
    private Timer t;

    /**
     * connection to close on inactivity timeout
     */
    private SessionDescriptor xmppSession;

    /**
     * duration to wait before closing connection
     */
    private long inactivityDuration;

    /**
     * true if timer is enabled
     */
    private boolean isRunning;

    /**
     * @param xmppSession
     *            the session to check
     * @param inactivityDuration
     *            delay between 2 checks
     * @param log
     *            logger
     */
    public InactivityChecker(final SessionDescriptor xmppSession,
            final long inactivityDuration, final Logger log) {
        this.xmppSession = xmppSession;
        this.inactivityDuration = inactivityDuration;
        this.log = log;
    }

    /**
     * wait for duration and then close the connection
     */
    public final void start() {
        this.t = new Timer();
        this.t.schedule(new CheckInactivityTask(), this.inactivityDuration,
                this.inactivityDuration);
        this.isRunning = true;
    }

    /**
     * stop the timer
     */
    public final void stop() {
        if (this.isRunning) {
            this.t.cancel();
        }
        this.isRunning = false;
    }

    /**
     * restart the timer
     */
    public final void restart() {
        this.stop();
        this.start();
    }

    /**
     * Task that close user's connection
     */
    class CheckInactivityTask extends TimerTask {

        /**
         * constructor
         */
        public CheckInactivityTask() {
            super();
        }

        /**
         * check if connection is inactive
         */
        @Override
        public void run() {
            final XMPPConnection connection = InactivityChecker.this.xmppSession
                    .getXmppConnection();
            if (connection == null) {
                return;
            }
            if ((connection.isConnected())
                    && (!(InactivityChecker.this.xmppSession.isListening()))) {
                InactivityChecker.this.log.info("Connection "
                        + connection.getUser() + " closed for inactivity");
                connection.disconnect();
                InactivityChecker.this.stop();
            }
        }
    }
}
