/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.binding.xmpp.listeners;

import java.util.Map;

import org.jivesoftware.smack.XMPPConnection;
import org.jivesoftware.smackx.muc.MultiUserChat;

/**
 * This descriptor handles information useful to connect to a xmpp server
 * @author mcarpentier - EBM Websourcing
 */
public class SessionDescriptor {

    /**
     * Hostname of the xmpp server
     */
    private String hostname;

    /**
     * Port of the xmpp server
     */
    private String port;

    /**
     * User
     */
    private SessionUser user;

    /**
     * Chatrooms joined by the user
     */
    private Map<String, MultiUserChat> rooms;
    /**
     * Session's connection to server
     */
    private XMPPConnection xmppConnection;

    /**
     * Session' s timeout checker
     */
    private InactivityChecker inactivityChecker;

    /**
     * Timeout for the session
     */
    private String sessionTimeout;

    /**
     * Timeout for the file transferts
     */
    private String transfertTimeout;
    /**
     * Msg Exchange default operation
     */
    private String operation;

    /**
     * Encoding format
     */
    private String encoding;

    /**
     * Decoding format
     */
    private String decoding;
    /**
     * Force message redirection
     */
    private String redirection;
    /**
     * packets listening
     */
    private String packets;
    /**
     * true if session is listening for incoming messages
     */
    private boolean isListening;
    /**
     * constructor
     */
    public SessionDescriptor() {
        super();
    }
    public final String getHostname() {
        return this.hostname;
    }

    public final void setHostname(final String hostname) {
        this.hostname = hostname;
    }

    public final String getPort() {
        return this.port;
    }

    public final void setPort(final String port) {
        this.port = port;
    }

    public final SessionUser getUser() {
        return this.user;
    }

    public final void setUser(final SessionUser user) {
        this.user = user;
    }

    public final Map<String, MultiUserChat> getRooms() {
        return this.rooms;
    }

    public final void setRooms(final Map<String, MultiUserChat> rooms) {
        this.rooms = rooms;
    }

    public final InactivityChecker getInactivityChecker() {
        return this.inactivityChecker;
    }

    public final void setInactivityChecker(final InactivityChecker inactivityChecker) {
        this.inactivityChecker = inactivityChecker;
    }

    public final XMPPConnection getXmppConnection() {
        return this.xmppConnection;
    }

    public final void setXmppConnection(final XMPPConnection xmppConnection) {
        this.xmppConnection = xmppConnection;
    }

    public final String getSessionTimeout() {
        return this.sessionTimeout;
    }

    public final void setSessionTimeout(final String sessionTimeout) {
        this.sessionTimeout = sessionTimeout;
    }

    public final String getTransfertTimeout() {
        return this.transfertTimeout;
    }

    public final void setTransfertTimeout(final String transfertTimeout) {
        this.transfertTimeout = transfertTimeout;
    }

    public final String getOperation() {
        return this.operation;
    }

    public final void setOperation(final String operation) {
        this.operation = operation;
    }

    public final String getDecoding() {
        return this.decoding;
    }

    public final void setDecoding(final String decoding) {
        this.decoding = decoding;
    }

    public final String getEncoding() {
        return this.encoding;
    }

    public final void setEncoding(final String encoding) {
        this.encoding = encoding;
    }

    public final String getRedirection() {
        return this.redirection;
    }

    public final void setRedirection(final String redirection) {
        this.redirection = redirection;
    }

    public final boolean isListening() {
        return this.isListening;
    }

    public final void setListening(final boolean listening) {
        this.isListening = listening;
    }

    public final String getPackets() {
        return this.packets;
    }

    public final void setPackets(final String packets) {
        this.packets = packets;
    }

}

