/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.binding.xmpp.listeners;

import java.io.File;
import java.util.Map;
import java.util.logging.Level;

import javax.jbi.JBIException;
import javax.jbi.messaging.MessagingException;

import org.jivesoftware.smack.packet.Message;
import org.objectweb.petals.binding.xmpp.XmppBCImpl;
import org.objectweb.petals.component.framework.listener.AbstractJBIListener;
import org.objectweb.petals.component.framework.util.Exchange;
import org.objectweb.petals.component.framework.util.Extensions;
import org.objectweb.petals.component.framework.util.StringHelper;

/**
 * Xmpp BC listener for incoming JBI messages
 * @author mcarpentier - EBM Websourcing
 */
public class XmppBCJBIListener extends AbstractJBIListener {

    protected SessionDescriptorBuilder sessionDescriptorBuilder;

    protected XmppSessionManager xmppConnectionManager;

    protected XmppMessageManager xmppMessageManager;
    /**
     * constructor
     */
    public XmppBCJBIListener() {
        super();
    }

    @Override
    protected final void init() {
        /**
         * init
         */
        this.sessionDescriptorBuilder = ((XmppBCImpl) this.getComponent())
                .getSessionDescriptorBuilder();

        this.xmppConnectionManager = ((XmppBCImpl) this.getComponent())
                .getXmppConnectionManager();

        this.xmppMessageManager = ((XmppBCImpl) this.getComponent())
                .getXmppMessageManager();
    }
    /**
     * Called by the JBI channel listener when a jbi message is accepted.
     * forward the message to the jabber server if the message contains an
     * attachment, send it via XMPP multiple attachments is not supported
     * @param exchange
     *            the {@link MessageExchangeWrapper} received
     */
    @Override
    public final void onJBIMessage(final Exchange exchange) {

        // Fault ignored
        if (exchange.getFault() == null) {
            try {
                this.getLogger().info("Message received");
                final Extensions extensions = new Extensions(this.getProvides()
                        .getExtensions());

                // Retrieve or init the xmpp session
                SessionDescriptor sessionDescriptor = this.xmppConnectionManager
                        .retrieveXmppSessionFromExtensions(extensions);

                if (sessionDescriptor == null) {
                    sessionDescriptor = this.sessionDescriptorBuilder
                            .build(extensions);
                    if ((sessionDescriptor.getUser() == null)
                            || (StringHelper.isNullOrEmpty(sessionDescriptor
                                    .getUser().getid()))) {
                        this.getLogger().log(Level.SEVERE,
                                "Couldn't init session - User is null");
                    }
                    this.xmppConnectionManager.getXmppSessions().put(
                            sessionDescriptor.getUser().getid(),
                            sessionDescriptor);
                    // deals with VCard information
                    if (((XmppBCImpl) this.getComponent()).getXmppContext() != null) {
                        final Map<String, String> map = ((XmppBCImpl) this
                                .getComponent()).getXmppContext()
                                .getSuRootPaths();
                        final String defaultAvatar = ((XmppBCImpl) this
                                .getComponent()).getDefaultResourcesPath()
                                + File.separator
                                + XmppBCImpl.PETALS_AVATAR_FILE;
                        this.sessionDescriptorBuilder.updateUserVcard(
                                sessionDescriptor, map, defaultAvatar);

                    }
                    this.sessionDescriptorBuilder.completeWithProvideProperties(
                            sessionDescriptor, extensions);
                    this.xmppConnectionManager.connectSession(sessionDescriptor);
                    // enable VCard support
                    this.xmppConnectionManager.handleVCard(sessionDescriptor);
                }
                else {
                    this.sessionDescriptorBuilder.completeWithProvideProperties(
                            sessionDescriptor, extensions);
                    this.xmppConnectionManager.connectSession(sessionDescriptor);
                }


                // Create a xmpp message with the incoming jbi message
                final Message message = this.xmppMessageManager
                        .mapNormalizedMessageToXmppMessage(exchange,
                                sessionDescriptor);

                // process message : send text or send file
                // Send this message
                this.xmppConnectionManager.processMessage(sessionDescriptor,
                        exchange, message);

            } catch (final JBIException e) {
                try {
                    if (!exchange.isInOnlyPattern()) {
                        exchange.setFault(new Exception(e));
                    } else {
                        this.getLogger().log(Level.SEVERE, e.getMessage(), e);
                    }
                } catch (final MessagingException e1) {
                    this.getLogger().log(Level.SEVERE,
                            "Can't return fault to consumer", e);
                }
            }
        }
    }

}
