/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.binding.xmpp.listeners;

import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.JBIException;
import javax.jbi.messaging.NormalizedMessage;

import org.jivesoftware.smack.Chat;
import org.jivesoftware.smack.ChatManager;
import org.jivesoftware.smack.PacketListener;
import org.jivesoftware.smack.XMPPConnection;
import org.jivesoftware.smack.XMPPException;
import org.jivesoftware.smack.packet.Packet;
import org.objectweb.petals.binding.xmpp.XmppBCImpl;
import org.objectweb.petals.binding.xmpp.util.XmppUtil;
import org.objectweb.petals.component.framework.MEPConstants;
import org.objectweb.petals.component.framework.util.Exchange;
import org.objectweb.petals.component.framework.util.SourceHelper;

/**
 * custom Xmpp packet listener
 * @author mcarpentier - EBM Websourcing
 */
public class XmppBCPacketListener implements PacketListener {

    private SessionDescriptor sessionDescriptor;

    private Logger logger;

    private String redirection;

    private String operation;

    private XmppBCExternalListener xmppBCExternalListener;

    public XmppBCPacketListener(final SessionDescriptor sessionDescriptor,
            final XmppBCExternalListener xmppBCExternalListener, final String redirection,
            final String operation, final Logger logger) {
        this.sessionDescriptor = sessionDescriptor;
        this.xmppBCExternalListener = xmppBCExternalListener;
        this.redirection = redirection;
        this.operation = operation;
        this.logger = logger;

    }

    public final Logger getLogger() {
        return this.logger;
    }

    public final SessionDescriptor getSessionDescriptor() {
        return this.sessionDescriptor;
    }

    public final void setSessionDescriptor(final SessionDescriptor sessionDescriptor) {
        this.sessionDescriptor = sessionDescriptor;
    }

    public final void processPacket(final Packet packet) {
        final String errorMsg = "Error while processing packet";
        this.getLogger().info("Message received from:" + packet.getFrom());
        try {
            final Exchange ex = this.xmppBCExternalListener.getXmppMessageManager()
                    .process(this.xmppBCExternalListener, this.redirection, packet,

                    this.operation, MEPConstants.IN_OPTIONAL_OUT_PATTERN);
            if (ex != null) {
                this.getLogger()
                        .info(
                                "Sending message to "
                                        + ex
                                                .getProperty(XmppBCImpl.DESTINATION_ADDRESS));
                this.xmppBCExternalListener.sendPacket(ex);

                final XMPPConnection con = this.sessionDescriptor.getXmppConnection();

                if ((con != null) && (con.isConnected())
                        && (ex.getOutMessage().getContent() != null)) {

                    final NormalizedMessage outMsg = ex.getOutMessage();
                    String msg = SourceHelper.createURLDecodedString(outMsg
                            .getContent(), "UTF-8");

                    msg = XmppUtil.getInstance().deleteRootTag(msg);

                    // send message via connection's chat manager
                    final ChatManager chatmanager = con.getChatManager();
                    final Chat newChat = chatmanager.createChat(packet.getFrom(),
                            null);
                    newChat.sendMessage(msg);

                }
            }
        } catch (final JBIException e) {
            this.getLogger().log(Level.SEVERE, errorMsg, e);
            this.getLogger().warning(e.getMessage());
        } catch (final XMPPException e) {
            this.getLogger().log(Level.SEVERE, errorMsg, e);
            this.getLogger().warning(e.getMessage());
        }

    }

}
