/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */


package org.objectweb.petals.binding.xmpp.listeners;

import java.util.Map;
import java.util.logging.Level;

import javax.jbi.JBIException;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.xml.transform.Source;

import org.objectweb.petals.binding.xmpp.util.XmppUtil;
import org.objectweb.petals.component.framework.PEtALSCDKException;
import org.objectweb.petals.component.framework.interceptor.Interceptor;
import org.objectweb.petals.component.framework.util.Exchange;
import org.objectweb.petals.component.framework.util.SourceHelper;
import org.objectweb.petals.jbi.descriptor.ServiceUnitExtensibleElement;

/**
 * 
 * Deals with message interceptors.
 *  Add root tag to sent message if needed
 * 
 * @author mcarpentier - EBM Websourcing
 * 
 */
public class XmppInterceptor extends Interceptor {

    public XmppInterceptor() {
        super();
    }
    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.petals.component.framework.interceptor.Interceptor#handleMessageAccept(org.objectweb.petals.component.framework.util.Exchange,
     *      org.objectweb.petals.jbi.descriptor.ServiceUnitExtensibleElement,
     *      java.util.Map)
     */
    @Override
    public void handleMessageAccept(final Exchange arg0,
            final ServiceUnitExtensibleElement arg1, final Map<String, String> arg2) {
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.petals.component.framework.interceptor.Interceptor#handleMessageSent(org.objectweb.petals.component.framework.util.Exchange,
     *      org.objectweb.petals.jbi.descriptor.ServiceUnitExtensibleElement,
     *      java.util.Map)
     */
    @Override
    public final void handleMessageSent(final Exchange arg0,
            final ServiceUnitExtensibleElement arg1, final Map<String, String> arg2) {
        final String errorMsg = "Error while intercepting packet";
        try {

            String jbiContent = arg0.getInMessageContentAsString();

            if (!XmppUtil.getInstance().isXmlWellFormed(jbiContent)) {
                jbiContent = XmppUtil.getInstance().addRootTag(jbiContent,
                        "root");
            }
            final NormalizedMessage nm = arg0.getInMessage();

            // Fill content
            Source source;
            try {
                source = SourceHelper.createSource(jbiContent);
            } catch (final PEtALSCDKException e) {
                throw new javax.jbi.messaging.MessagingException(
                        "Can't create source from String content", e);
            }
            nm.setContent(source);
        } catch (final MessagingException e) {
            this.logger.log(Level.SEVERE, errorMsg, e);
            this.logger.warning(e.getMessage());
        } catch (final JBIException e) {
            this.logger.log(Level.SEVERE, errorMsg, e);
            this.logger.warning(e.getMessage());
        }

    }
    /* (non-Javadoc)
     * @see org.objectweb.petals.component.framework.interceptor.Interceptor#handleMessageAcceptResponse(org.objectweb.petals.component.framework.util.Exchange, org.objectweb.petals.jbi.descriptor.ServiceUnitExtensibleElement, java.util.Map)
     */
    @Override
    public void handleMessageAcceptResponse(Exchange arg0, ServiceUnitExtensibleElement arg1,
            Map<String, String> arg2) {
        // TODO Auto-generated method stub
        
    }
    /* (non-Javadoc)
     * @see org.objectweb.petals.component.framework.interceptor.Interceptor#handleMessageSentResponse(org.objectweb.petals.component.framework.util.Exchange, org.objectweb.petals.jbi.descriptor.ServiceUnitExtensibleElement, java.util.Map)
     */
    @Override
    public void handleMessageSentResponse(Exchange arg0, ServiceUnitExtensibleElement arg1,
            Map<String, String> arg2) {
        // TODO Auto-generated method stub
        
    }

}
